const { Pool } = require('pg');

const dbConfig = {
    user: process.env.POSTGRES_USER || 'e2ee',
    password: process.env.POSTGRES_PASSWORD,
    host: process.env.POSTGRES_HOST || '127.0.0.1',
    port: process.env.POSTGRES_PORT || '5432',
    database: process.env.POSTGRES_DB || 'e2ee',
};

const pool = new Pool(dbConfig);

const database = {
    init: async () => {
        while (true) {
            try {
                await database.checkIfDatabaseIsUp();
                console.log('Database is up, initializing...');
                database.checkSchema().then((exists) => {
                    if (!exists) {
                        database.createTables();
                    } else {
                        console.log("Database schema is already correct.");
                    }
                }).catch((err) => {
                    console.error("Error checking schema:", err);
                });
                break;
            } catch (err) {
                console.error('Waiting for the database to start...');
            }
            await new Promise(resolve => setTimeout(resolve, 5000));
        }
    },
    createTables: () => {
        pool.query(`
            CREATE TABLE IF NOT EXISTS "users" (
                uuid integer PRIMARY KEY,
                pubkey text
            );
        `, (err, _) => {
            if (err) {
                console.error('Error creating users table', err);
                return;
            }
            pool.query(`
                CREATE SEQUENCE IF NOT EXISTS uuid_sequence
                INCREMENT BY 1
                START WITH 1;
            `, (err, _) => {
                if (err) {
                    console.error('Error creating sequence', err);
                } else {
                    console.log("users table and sequence created successfully.");
                }
            });
        });
    },
    checkSchema: () => {
        return new Promise((resolve, reject) => {
            pool.query(`
                SELECT EXISTS (
                    SELECT 1 FROM pg_tables WHERE tablename = 'users'
                ) AS table_exists;
            `, (err, res) => {
                if (err) {
                    console.error('Error executing query', err);
                    reject(err);
                } else {
                    const tableExists = res.rows[0].table_exists;
                    resolve(tableExists);
                }
            });
        });
    },
    checkIfDatabaseIsUp: () => {
        return new Promise((resolve, reject) => {
            pool.query('SELECT 1', (err, res) => {
                if (err) {
                    reject(false);
                } else {
                    resolve(true);
                }
            });
        });
    },
    addUser: async (pubkey) => {
        if (!pubkey) {
            console.error("Pubkey is required");
            return;
        }
        try {
            const result = await pool.query('SELECT NEXTVAL(\'uuid_sequence\') AS next_uuid');
            const nextUuid = result.rows[0].next_uuid;
            await pool.query(
                'INSERT INTO "users" (uuid, pubkey) VALUES ($1, $2)',
                [nextUuid, pubkey]
            );
            console.log(`Added user with the public key ${pubkey} .`);
        } catch (err) {
            console.error('Error adding user:', err);
        }
    },
    getPublicKeys: async () => {
        try {
            const result = await pool.query('SELECT pubkey FROM users');
            const publicKeys = result.rows.map(row => row.pubkey);
            return publicKeys;
        } catch (err) {
            console.error('Error retrieving public keys:', err);
            throw new Error('Error retrieving public keys');
        }
    }
};

module.exports = database;
